##############################################################
#
#   Humerus - Example game, basic
#
#   Very minimal single-player pong-like game.
#   Press mouse button to serve ball. You get a point
#   each time you hit the ball, lose one when you miss.
#   Earn 3 points to win the game.
#
##############################################################

#
#  This is so you can run the example from the source
#  directory without installing humerus.
#

import os, sys
d = os.path.dirname
sys.path.insert(1, d(d(d(os.path.abspath(sys.argv[0])))))

#
#  Parse command line options and initialize display
#  before doing anything else.
#

from humerus.options import Options

class PongOptions(Options):

	frame_rate_option = 20
	screen_size_option = (640, 480)

options = PongOptions()

#
#  Now do the rest.
#

import random
from pygame import Rect
from albow.screen import Screen
from albow.controls import Label
from humerus.hgame import HGame
from humerus.hshell import HShell
from humerus.main_commands import MainCommandScreen

class State(object):
	
	def __init__(self):
		self.score = 0
		self.ball_position = None
		self.is_completed = False
	
	def begin_frame(self, court_size):
		if self.score >= 3:
			self.is_completed = True
		p = self.ball_position
		if p:
			x, y = p
			vx, vy = self.ball_speed
			w, h = court_size
			x += vx
			y += vy
			if x < 0:
				x = -x
				vx = -vx
			elif x > w:
				r = game.bat_rect
				if r and not (y >= r.top and y <= r.bottom):
					self.ball_position = None
					self.score -= 1
					return
				else:
					self.score += 1
					x = w - (x - w)
					vx = -vx
			if y < 0:
				y = -y
				vy = -vy
			elif y > h:
				y = h - (y - h)
				vy = -vy
			self.ball_position = x, y
			self.ball_speed = vx, vy

class PongGame(HGame):

	save_file_suffix = ".pongam"
	level_file_suffix = ".ponglev"
	level_set_suffix = ".pongset"

	bat_size = 50
	bat_rect = None
	
	def new_state(self):
		return State()
	
	def game_is_completed(self):
		result = self.state.is_completed
		return result
	
	def begin_frame(self):
		self.state.begin_frame(shell.size)

help_text = \
"""Click the mouse to serve.\n
Hit the ball to gain a point.\n
You lose a point when you miss.\n
Get 3 points to win."""

class PongHelpScreen(Screen):

	def __init__(self, shell):
		Screen.__init__(self, shell)
		help = Label(help_text)
		self.add_centered(help)

class PongPlayScreen(MainCommandScreen):

	ball_color = (255, 255, 255)
	ball_size = (8, 8)
	bat_color = (255, 255, 255)
	bat_width = 8

	def draw(self, surf):
		state = game.state
		ball = state.ball_position
		r = self.ball_rect()
		if r:
			surf.fill(self.ball_color, r)
		r = game.bat_rect
		if r:
			surf.fill(self.bat_color, r)
		s = "Score: %s" % state.score
		buf = self.font.render(s, True, self.fg_color)
		r = buf.get_rect()
		r.midbottom = self.width // 2, self.height
		surf.blit(buf, r)
	
	def ball_rect(self):
		p = game.state.ball_position
		if p:
			r = Rect((0, 0), self.ball_size)
			r.center = p
			return r
	
	def begin_frame(self):
		e = self.get_mouse()
		x, y = e.local
		r = Rect(0, 0, self.bat_width, game.bat_size)
		r.midright = self.width, y
		game.bat_rect = r
		self.invalidate()
	
	def mouse_down(self, e):
		state = game.state
		if state.ball_position is None:
			r = game.bat_rect
			if r:
				state.ball_position = r.midleft
				vx = -10
				vy = random.randint(-10, 10)
				state.ball_speed = vx, vy
		
class PongShell(HShell):

	game_title = "Python Pong"

	def __init__(self, game, options):
		HShell.__init__(self, game, options)
		self.about_screen = PongHelpScreen(self)
		self.main_menu_screen = MainCommandScreen(self, with_menu = True)
		self.play_screen = PongPlayScreen(self)


game = PongGame()
shell = PongShell(game, options)
shell.run()
